package com.scansolutions.mrzflutterplugin

import android.app.Activity
import android.graphics.Bitmap
import androidx.annotation.NonNull
import com.scansolutions.mrzscannerlib.MRZResultModel
import com.scansolutions.mrzscannerlib.MRZScanner
import com.scansolutions.mrzscannerlib.MRZScannerListener
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

/** MrzflutterpluginPlugin */
public class MrzflutterpluginPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private var mContext: Activity? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "mrzflutterplugin")
        channel.setMethodCallHandler(this);
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "mrzflutterplugin")
            channel.setMethodCallHandler(MrzflutterpluginPlugin())
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "startScanner" -> {
                handleStartScanner(result)
            }
            "registerWithLicenceKey" -> {
                handleRegisterWithLicenceKey(call)
            }
            "setDateFormat" -> {
                handleSetDateFormat(call)
            }
            "scanFromGallery" -> {
                handleScanFromGallery(result)
            }
            "getSdkVersion" -> {
                result.success(MRZScanner.sdkVersion())
            }
            "setIDActive" -> {
                if (call.arguments is Boolean) {
                    MRZScanner.setIDActive(call.arguments as Boolean)
                }
            }
            "setPassportActive" -> {
                if (call.arguments is Boolean) {
                    MRZScanner.setPassportActive(call.arguments as Boolean)
                }
            }
            "setVisaActive" -> {
                if (call.arguments is Boolean) {
                    MRZScanner.setVisaActive(call.arguments as Boolean)
                }
            }

            else -> {
                result.notImplemented()
            }
        }
    }

    private fun handleScanFromGallery(result: Result) {
        if (mContext != null) {
            MRZScanner.scanFromGallery(mContext, object : MRZScannerListener {
                override fun scanImageFailed() {
                    result.error("scanImageFailed", "scanImageFailed", null)
                }

                override fun permissionsWereDenied() {
                    result.error("permissionsWereDenied", "permissionsWereDenied", null)
                }

                override fun successfulScanWithDocumentImage(p0: Bitmap?) {
                }

                override fun successfulScanWithResult(p0: MRZResultModel?) {
                    result.success(p0?.toJSON().toString())
                }

            })
        }
    }

    private fun handleSetDateFormat(call: MethodCall) {
        if (call.arguments is String) {
            val dateFormat = call.arguments as String
            MRZScanner.setDateFormat(dateFormat)
        }
    }

    private fun handleRegisterWithLicenceKey(call: MethodCall) {
        if (mContext != null && call.arguments is String) {
            val licenceKey = call.arguments as String
            MRZScanner.registerWithLicenseKey(mContext, licenceKey)
        }
    }

    private fun handleStartScanner(result: Result) {
        if (mContext != null) {
            ScannerActivity.startScanner(mContext, object : MRZScannerListener {
                override fun scanImageFailed() {
                    result.error("scanImageFailed", "scanImageFailed", null)
                }

                override fun permissionsWereDenied() {
                    result.error("permissionsWereDenied", "permissionsWereDenied", null)
                }

                override fun successfulScanWithDocumentImage(p0: Bitmap?) {
                }

                override fun successfulScanWithResult(p0: MRZResultModel?) {
                    result.success(p0?.toJSON().toString())
                }

            }) {
                result.error("scannerWasDismissed", "scannerWasDismissed", null)
            }

        } else {
            result.error("context == null", null, null)
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onDetachedFromActivity() {
        mContext = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        mContext = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }
}
